@@ -30,6 +30,7 @@ from time import perf_counter
 
 import numpy as np
 from scipy.sparse.linalg import spsolve
+from scipy.sparse import csr_matrix
 
 from ..math import norm
 from .._assembly import IntegralFormMixed
@@ -46,24 +47,27 @@ class Result:
         self.iterations = iterations
 
 
-def fun_bodies(bodies, x=None, parallel=False, jit=False):
+def fun_items(items, x, parallel=False, jit=False):
     "Force residuals from assembly of equilibrium (weak form)."
 
     # init keyword arguments
     kwargs = {"parallel": parallel, "jit": jit}
 
-    # assemble vector of first body
-    vector = bodies[0].assemble.vector(field=x, **kwargs)
+    # link field of items with global field
+    [item.field.link(x) for item in items]
 
-    # loop over other bodies
-    for body in bodies[1:]:
+    # init vector with shape from global field
+    shape = (np.sum(x.fieldsizes), 1)
+    vector = csr_matrix(shape)
+
+    for body in items:
 
         # assemble vector
-        r = body.assemble.vector(field=x, **kwargs)
+        r = body.assemble.vector(field=body.field, **kwargs)
 
         # check and reshape vector
-        if r.shape != vector.shape:
-            r.resize(*vector.shape)
+        if r.shape != shape:
+            r.resize(*shape)
 
         # add vector
         vector += r
@@ -71,24 +75,24 @@ def fun_bodies(bodies, x=None, parallel=False, jit=False):
     return vector.toarray()[:, 0]
 
 
-def jac_bodies(bodies, parallel=False, jit=False):
+def jac_items(items, x, parallel=False, jit=False):
     "Tangent stiffness matrix from assembly of linearized equilibrium."
 
     # init keyword arguments
     kwargs = {"parallel": parallel, "jit": jit}
 
-    # assemble matrix of first body
-    matrix = bodies[0].assemble.matrix(**kwargs)
+    # init matrix with shape from global field
+    shape = (np.sum(x.fieldsizes), np.sum(x.fieldsizes))
+    matrix = csr_matrix(shape)
 
-    # loop over other bodies
-    for body in bodies[1:]:
+    for body in items:
 
         # assemble matrix
         K = body.assemble.matrix(**kwargs)
 
         # check and reshape matrix
         if K.shape != matrix.shape:
-            K.resize(*matrix.shape)
+            K.resize(*shape)
 
         # add matrix
         matrix += K
@@ -130,9 +134,32 @@ def solve(A, b, x, dof1, dof0, offsets=None, ext0=None, solver=spsolve):
     return dx
 
 
-def check(dx, x, f, tol):
+def check(dx, x, f, xtol, ftol, dof1=None, dof0=None, items=None, eps=1e-3):
     "Check result."
-    return np.sum(norm(dx)), np.all(norm(dx) < tol)
+
+    sumnorm = lambda x: np.sum(norm(x))
+    xnorm = sumnorm(dx)
+
+    if dof1 is None:
+        dof1 = slice(None)
+
+    if dof0 is None:
+        dof0 = slice(0, 0)
+
+    fnorm = sumnorm(f[dof1]) / (eps + sumnorm(f[dof0]))
+    success = fnorm < ftol  # and xnorm < xtol
+
+    if success and items is not None:
+        for item in items:
+            [item.results.update_statevars() for item in items]
+
+    return xnorm, fnorm, success
+
+
+def update(x, dx):
+    "Update field."
+    # x += dx # in-place
+    return x + dx
 
 
 def newtonrhapson(
@@ -141,15 +168,15 @@ def newtonrhapson(
     jac=jac,
     solve=solve,
     maxiter=16,
-    update=lambda x, dx: x + dx,
+    update=update,
     check=check,
     args=(),
     kwargs={},
-    kwargs_solve={},
-    kwargs_check={},
+    kwargs_solve=None,
+    kwargs_check=None,
     tol=np.sqrt(np.finfo(float).eps),
     umat=None,
-    bodies=None,
+    items=None,
     dof1=None,
     dof0=None,
     ext0=None,
@@ -194,15 +221,21 @@ def newtonrhapson(
         # copy x0
         x = x0
     else:
-        # copy field of body
-        x = bodies[0].field
+        # obtain field of first body
+        x = items[0].field
 
     if umat is not None:
         kwargs["umat"] = umat
 
+    if kwargs_solve is None:
+        kwargs_solve = {}
+
+    if kwargs_check is None:
+        kwargs_check = {}
+
     # pre-evaluate function at given unknowns "x"
-    if bodies is not None:
-        f = fun_bodies(bodies, x, *args, **kwargs)
+    if items is not None:
+        f = fun_items(items, x, *args, **kwargs)
     else:
         f = fun(x, *args, **kwargs)
 
@@ -211,15 +244,15 @@ def newtonrhapson(
         print("Newton-Rhapson solver")
         print("=====================")
         print()
-        print("| # |  norm(dx) |")
-        print("|---|-----------|")
+        print("| # | norm(fun) |  norm(dx) |")
+        print("|---|-----------|-----------|")
 
     # iteration loop
     for iteration in range(maxiter):
 
         # evaluate jacobian at unknowns "x"
-        if bodies is not None:
-            K = jac_bodies(bodies, *args, **kwargs)
+        if items is not None:
+            K = jac_items(items, x, *args, **kwargs)
         else:
             K = jac(x, *args, **kwargs)
 
@@ -238,23 +271,25 @@ def newtonrhapson(
         x = update(x, dx)
 
         # evaluate function at unknowns "x"
-        if bodies is not None:
-            f = fun_bodies(bodies, x, *args, **kwargs)
+        if items is not None:
+            f = fun_items(items, x, *args, **kwargs)
         else:
             f = fun(x, *args, **kwargs)
 
         # check success of solution
-        norm, success = check(dx, x, f, tol, **kwargs_check)
+        xnorm, fnorm, success = check(
+            dx, x, f, tol, tol, dof1, dof0, items, **kwargs_check
+        )
 
         if verbose:
-            print("|%2d | %1.3e |" % (1 + iteration, norm))
+            print("|%2d | %1.3e | %1.3e |" % (1 + iteration, fnorm, xnorm))
 
         if success:
-            if not timing:
+            if verbose and not timing:
                 print("\nSolution converged in %d iterations.\n" % (iteration + 1))
             break
 
-        if np.isnan(norm):
+        if np.any(np.isnan([xnorm, fnorm])):
             raise ValueError("Norm of unknowns is NaN.")
 
     if 1 + iteration == maxiter and not success:
@@ -265,7 +300,7 @@ def newtonrhapson(
     if export_jac:
         Res.jac = K
 
-    if timing:
+    if verbose and timing:
         time_finish = perf_counter()
         print(
             "\nSolution converged in %d iterations within %1.4g seconds.\n"
