@@ -63,6 +63,9 @@ def test_meshes():
     assert m.points.shape == (5, 1)
     assert m.cells.shape == (4, 2)
 
+    mr = fe.mesh.revolve(m, n=11, phi=180, axis=2)
+    assert mr.ncells == 4 * 10
+
     m = fe.Rectangle(a=(-1.2, -2), b=(2, 3.1), n=(4, 9))
     assert m.points.shape == (4 * 9, 2)
     assert m.cells.shape == (3 * 8, 4)
